package mvd.jester.tests;

import java.lang.reflect.Constructor;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import mvd.jester.model.SystemSetup;
import mvd.jester.utils.Logger;

/**
 * TestEnvironment
 */
public class TestEnvironment {

    private final long numberOfTaskSets;
    private final SystemSetup systemSetup;
    private final SystemSetup.Builder builder;
    private final Set<Constructor<? extends AbstractTest>> abstractTests;

    public TestEnvironment(SystemSetup.Builder builder, long numberOfTaskSets) {
        this.numberOfTaskSets = numberOfTaskSets;
        abstractTests = new HashSet<>();

        this.builder = builder;
        this.systemSetup = builder.build();
    }

    public TestEnvironment registerTestInterface(Class<? extends AbstractTest> abstractTest) {
        try {
            abstractTests.add(abstractTest.getConstructor(SystemSetup.class));
        } catch (Exception e) {
            System.out.println("Missing constructor!");
        }

        return this;
    }

    public void runTests() {
        Set<AbstractTest> testCases = new HashSet<>();

        Map<Long, Map<AbstractTest, Long>> results = new HashMap<>();

        for (Constructor<? extends AbstractTest> t : abstractTests) {
            try {
                testCases.add(t.newInstance(this.systemSetup));
            } catch (Exception e) {
                System.out.println("Could not instantiate object of AbstractTest!");
            }
        }

        long checkedTasksets = 0;

        while (checkedTasksets < numberOfTaskSets) {
            builder.rebuild(this.systemSetup);

            double utilization = this.systemSetup.getUtilization();

            while (utilization <= this.systemSetup.getNumberOfProcessors()
                    && checkedTasksets < numberOfTaskSets) {
                checkedTasksets++;

                long roundedUtilization = (long) (utilization * 10);
                for (AbstractTest t : testCases) {
                    if (t.runSchedulabilityCheck()) {
                        results.computeIfAbsent(roundedUtilization,
                                k -> new HashMap<AbstractTest, Long>())
                                .compute(t, (k, v) -> (v == null) ? 1 : v + 1);
                    }
                }
                builder.addTask(systemSetup);

                utilization = this.systemSetup.getUtilization();
            }
        }

        logResults(testCases, results);
    }

    private void logResults(Set<AbstractTest> testCases,
            Map<Long, Map<AbstractTest, Long>> results) {
        LocalTime date = LocalTime.now();
        Logger log = new Logger("./results/results_" + systemSetup.getNumberOfProcessors() + "_"
                + date.getHour() + ":" + date.getMinute() + ".txt");
        String firstLine = new String("Utilization");

        for (AbstractTest t : testCases) {
            firstLine = firstLine + "\t" + t.getName();
        }

        log.log(firstLine);

        for (Long util : results.keySet()) {
            String line = String.valueOf((double) util / 10);
            Map<AbstractTest, Long> tests = results.get(util);

            for (AbstractTest t : testCases) {
                if (tests.containsKey(t)) {
                    line += "\t" + tests.get(t);
                } else {
                    line += "\t" + "0";
                }
            }
            log.log(line);
        }

        log.finalize();
    }
}
